local function GetRequiredLevel(itemLink, itemType)
  return itemType ~= ITEMTYPE_RECIPE and GetItemLinkRequiredLevel(itemLink) or 1
end

local function GetVirtualIndex(itemLink)
  local itemType = GetItemLinkItemType(itemLink)
  local itemId = GetItemLinkItemId(itemLink)
  local requiredLevel = GetRequiredLevel(itemLink, itemType)
  local requiredChampionPoints = GetItemLinkRequiredChampionPoints(itemLink) / 10
  local quality = GetItemLinkDisplayQuality(itemLink)
  local index = itemId .. ":" .. requiredLevel .. ":" .. requiredChampionPoints .. ":" .. quality
  return index
end

local function IsItemLinkCraftedGlyph(itemLink)
  local itemType = GetItemLinkItemType(itemLink)
  local isCrafted = IsItemLinkCrafted(itemLink)
  local isGlyph = itemType == ITEMTYPE_GLYPH_ARMOR or itemType == ITEMTYPE_GLYPH_JEWELRY or itemType == ITEMTYPE_GLYPH_WEAPON
  if isCrafted and isGlyph then
    return true
  end
  return false
end

function LibRecipe:GetRecipeItemLink(itemLink)
  if IsItemLinkCraftedGlyph(itemLink) then
    return itemLink
  end
  local quality = GetItemLinkDisplayQuality(itemLink)
  local itemId = GetItemLinkItemId(itemLink)
  local offset = LibRecipe.internalRecipeData[itemId]
  if offset == nil then return end
  local recipeId = itemId + offset
  quality = quality + 1
  return string.format("|H1:item:%d:%d:1:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h", recipeId, quality)
end

function LibRecipe:GetItemLinkFromItemId(itemId)
  return string.format("|H1:item:%d:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h", itemId)
end

function LibRecipe:GetNumIngredients(itemLink)
  itemLink = LibRecipe:GetRecipeItemLink(itemLink)
  if IsItemLinkCraftedGlyph(itemLink) then
    local virtualIndex = GetVirtualIndex(itemLink)
    if LibRecipe.virtualRecipe[virtualIndex] then
      return #LibRecipe.virtualRecipe[virtualIndex]
    end
    return 0
  end
  local itemType = GetItemLinkItemType(itemLink)
  if not itemLink or itemType ~= ITEMTYPE_RECIPE then return 0 end
  return GetItemLinkRecipeNumIngredients(itemLink)
end

function LibRecipe:GetIngredientInfoByIndex(itemLink, index)
  itemLink = LibRecipe:GetRecipeItemLink(itemLink)
  if IsItemLinkCraftedGlyph(itemLink) then
    local virtualIndex = GetVirtualIndex(itemLink)
    if LibRecipe.virtualRecipe[virtualIndex] then
      local ingredientItemLink = LibRecipe.virtualRecipe[virtualIndex][index].item
      local displayQuality = GetItemLinkDisplayQuality(ingredientItemLink)
      local ingredientName = GetItemLinkName(ingredientItemLink)
      local amountRequired = LibRecipe.virtualRecipe[virtualIndex][index].required
      return ingredientItemLink, ingredientName, displayQuality, amountRequired
    end
    return nil, nil, nil, nil
  end
  local ingredientItemLink = GetItemLinkRecipeIngredientItemLink(itemLink, index, LINK_STYLE_DEFAULT)
  if ingredientItemLink == '' then return nil, nil, nil, nil end
  local displayQuality = GetItemLinkDisplayQuality(ingredientItemLink)
  local ingredientName, amountInInventoryAndBank, amountRequired = GetItemLinkRecipeIngredientInfo(itemLink, index)
  return ingredientItemLink, ingredientName, displayQuality, amountRequired
end

-- 1 == positive potency 2 == negative potency
-- 1 == Additive, 2 == Subtractive
local effectIds = {
  [43573] = { 45831, 2 },
  [26580] = { 45831, 1 },
  [45868] = { 45832, 2 },
  [26582] = { 45832, 1 },
  [45867] = { 45833, 2 },
  [26588] = { 45833, 1 },
  [26581] = { 45834, 1 },
  [45869] = { 45834, 2 },
  [26583] = { 45835, 1 },
  [45870] = { 45835, 2 },
  [26589] = { 45836, 1 },
  [45871] = { 45836, 2 },
  [26586] = { 45837, 2 },
  [26587] = { 45837, 1 },
  [26848] = { 45838, 1 },
  [26849] = { 45838, 2 },
  [5365] = { 45839, 1 },
  [5364] = { 45839, 2 },
  [43570] = { 45840, 2 },
  [26844] = { 45840, 1 },
  [26841] = { 45841, 1 },
  [26847] = { 45841, 2 },
  [26845] = { 45842, 2 },
  [5366] = { 45842, 1 },
  [54484] = { 45843, 1 },
  [26591] = { 45843, 2 },
  [45874] = { 45846, 1 },
  [45875] = { 45846, 2 },
  [45885] = { 45847, 2 },
  [45883] = { 45847, 1 },
  [45886] = { 45848, 2 },
  [45884] = { 45848, 1 },
  [45873] = { 45849, 2 },
  [45872] = { 45849, 1 },
  [166046] = { 166045, 2 },
  [166047] = { 166045, 1 },
  [68344] = { 68342, 2 },
  [68343] = { 68342, 1 },
}

local potencyIds = {
  [5] = { 45855, 45817 },
  [10] = { 45856, 45818 },
  [15] = { 45857, 45819 },
  [20] = { 45806, 45820 },
  [25] = { 45807, 45821 },
  [30] = { 45808, 45822 },
  [35] = { 45809, 45823 },
  [40] = { 45810, 45824 },
  [45] = { 45811, 45825 },
  [51] = { 45812, 45826 },
  [53] = { 45813, 45827 },
  [55] = { 45814, 45828 },
  [57] = { 45815, 45829 },
  [63] = { 45816, 45830 },
  [65] = { 64509, 64508 },
  [66] = { 68341, 68340 },
}

-- aspect rune ID numbers
local qualityIds = {
  [1] = 45850,
  [2] = 45851,
  [3] = 45852,
  [4] = 45853,
  [5] = 45854,
}

local function splitItem(t, index)
  local itemId, itemLevel, itemCPLevel, itemQuality = zo_strmatch(index, "(%d+):(%d+):(%d+):(%d+)")
  itemId = tonumber(itemId)
  itemLevel = tonumber(itemLevel)
  itemCPLevel = tonumber(itemCPLevel)
  itemQuality = tonumber(itemQuality)
  local combinedLvl = itemLevel + itemCPLevel
  local essence = effectIds[itemId]
  if not essence then return end
  if not potencyIds[combinedLvl] then return end
  local potency = potencyIds[combinedLvl][essence[2]]
  local quality = qualityIds[itemQuality]
  if not potency or not quality or not essence then return end
  return {
    { ["item"] = LibRecipe:GetItemLinkFromItemId(potency), ["required"] = 1 },
    { ["item"] = LibRecipe:GetItemLinkFromItemId(essence[1]), ["required"] = 1 },
    { ["item"] = LibRecipe:GetItemLinkFromItemId(quality), ["required"] = 1 }
  }
end

LibRecipe.virtualRecipe = {}
setmetatable(LibRecipe.virtualRecipe, { ["__index"] = splitItem })

-- |H1:item:33837:3:1:0:0:0:0:0:0:0:0:0:0:0:0:0:1:0:0:0:0|h|h Baked Apples
-- |H1:item:45889:3:1:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h Recipe: Baked Apples
-- |H1:item:26588:370:50:0:0:0:0:0:0:0:0:0:0:0:0:0:1:0:0:0:0|h|h Crafted Glyph
-- |H1:item:126359:3:1:0:0:0:0:0:0:0:0:0:0:0:0:0:1:0:0:0:0|h|h Crafted Furnishing

-- /script d(LibRecipe:GetItemLinkFromItemId(33837))
-- /script d(LibRecipe:GetNumIngredients("|H1:item:33837:3:1:0:0:0:0:0:0:0:0:0:0:0:0:0:1:0:0:0:0|h|h"))
-- /script d(LibRecipe:GetIngredientInfoByIndex("|H1:item:33837:3:1:0:0:0:0:0:0:0:0:0:0:0:0:0:1:0:0:0:0|h|h", 1))
-- /script d(GetItemLinkRecipeIngredientItemLink("|H1:item:126359:3:1:0:0:0:0:0:0:0:0:0:0:0:0:0:1:0:0:0:0|h|h", 1, LINK_STYLE_DEFAULT))
