local function GetItemLinkFromItemId(itemId)
  return string.format("|H1:item:%d:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h", itemId)
end

function LibRecipe:LoadRecipesFrom(endNumber)
  local resultLink
  for recNumber = 1, endNumber do
    local itemLink = GetItemLinkFromItemId(recNumber)
    local itemType = GetItemLinkItemType(itemLink)

    if itemType == ITEMTYPE_ENCHANTING_RUNE_ESSENCE then
      table.insert(LibRecipe.essenceRunes, recNumber)
    elseif itemType == ITEMTYPE_ENCHANTING_RUNE_POTENCY then
      table.insert(LibRecipe.potencyRunes, recNumber)
    elseif itemType == ITEMTYPE_ENCHANTING_RUNE_ASPECT then
      table.insert(LibRecipe.aspectRunes, recNumber)
    elseif itemType == ITEMTYPE_POTION_BASE then
      local levelRequired = GetItemLinkRequiredLevel(itemLink) + GetItemLinkRequiredChampionPoints(itemLink)
      LibRecipe.potionSolvents[levelRequired] = recNumber
      LibRecipe.potionSolventsItemLinks[recNumber] = LibRecipe.potionSolventsItemLinks[recNumber] or {}
      LibRecipe.potionSolventsItemLinks[recNumber][levelRequired] = LibRecipe.potionSolventsItemLinks[recNumber][levelRequired] or {}
      LibRecipe.potionSolventsItemLinks[recNumber][levelRequired][1] = itemLink
    elseif itemType == ITEMTYPE_POISON_BASE then
      local levelRequired = GetItemLinkRequiredLevel(itemLink) + GetItemLinkRequiredChampionPoints(itemLink)
      LibRecipe.poisonSolvents[levelRequired] = recNumber
      LibRecipe.poisonSolventsItemLinks[recNumber] = LibRecipe.poisonSolventsItemLinks[recNumber] or {}
      LibRecipe.poisonSolventsItemLinks[recNumber][levelRequired] = LibRecipe.poisonSolventsItemLinks[recNumber][levelRequired] or {}
      LibRecipe.poisonSolventsItemLinks[recNumber][levelRequired][2] = itemLink
    elseif itemType == ITEMTYPE_REAGENT then
      table.insert(LibRecipe.reagents, recNumber)
      LibRecipe.reagentItemLinks[recNumber] = LibRecipe.reagentItemLinks[recNumber] or {}
      LibRecipe.reagentItemLinks[recNumber][2] = itemLink
      --[[
      LibRecipe.reagents[recNumber] = {}
      for i = 1, GetMaxTraits() do
          local _, traitName = GetItemLinkReagentTraitInfo(itemLink, i)
          table.insert(LibRecipe.reagents[recNumber], traitName)
          -- If you get an error here, you don't know all the flower/rune traits....
          LibRecipe.traits[traitName] = LibRecipe.traits[traitName] or {}
          table.insert(LibRecipe.traits[traitName], recNumber)
      end
      --]]
    elseif itemType == ITEMTYPE_RECIPE then
      resultLink = GetItemLinkRecipeResultItemLink(itemLink)
      if (resultLink ~= MM_STRING_EMPTY) then
        LibRecipe.recipeData[resultLink] = itemLink
        LibRecipe.recipeCount = LibRecipe.recipeCount + 1
        --debug
        --d(LibRecipe.recipeCount .. ') ' .. itemLink .. ' --> ' .. resultLink  .. ' ('  .. recNumber .. ')')
      end
    end
  end

  -- LibRecipe:dm("Info", '|cFFFF00Recipes Initialized -- Found information on ' .. LibRecipe.recipeCount .. ' recipes.|r')
  LibRecipe_SavedVariables.recipeData = LibRecipe.recipeData
  LibRecipe_SavedVariables.essenceRunes = LibRecipe.essenceRunes
  LibRecipe_SavedVariables.potencyRunes = LibRecipe.potencyRunes
  LibRecipe_SavedVariables.aspectRunes = LibRecipe.aspectRunes
  LibRecipe_SavedVariables.reagents = LibRecipe.reagents
  LibRecipe_SavedVariables.potionSolvents = LibRecipe.potionSolvents
  LibRecipe_SavedVariables.poisonSolvents = LibRecipe.poisonSolvents
  LibRecipe_SavedVariables.reagentItemLinks = LibRecipe.reagentItemLinks
  LibRecipe_SavedVariables.potionSolventsItemLinks = LibRecipe.potionSolventsItemLinks
  LibRecipe_SavedVariables.poisonSolventsItemLinks = LibRecipe.poisonSolventsItemLinks
end

-- used to generate recipie table
function LibRecipe:ParseTable()
  LibRecipe_SavedVariables.parseTable = {}
  local linkTable = LibRecipe_SavedVariables.recipeData
  for k, v in pairs(linkTable) do
    local leftValue = GetItemLinkItemId(k)
    local rightValue = GetItemLinkItemId(v)
    local result = rightValue - leftValue
    local stringResult = string.format("[%s] = %s,", leftValue, result)
    table.insert(LibRecipe_SavedVariables.parseTable, stringResult)
  end
end

function LibRecipe:SetupRecipeInfo()
  if not LibRecipe.recipeData then
    LibRecipe.recipeData = {}
    LibRecipe.recipeCount = 0

    LibRecipe.essenceRunes = {}
    LibRecipe.aspectRunes = {}
    LibRecipe.potencyRunes = {}

    LibRecipe.virtualRecipe = {}
    LibRecipe.virtualRecipeCount = 0

    LibRecipe.traits = {}
    LibRecipe.reagents = {}
    LibRecipe.potionSolvents = {}
    LibRecipe.poisonSolvents = {}
    LibRecipe.reagentItemLinks = {}
    LibRecipe.potionSolventsItemLinks = {}
    LibRecipe.poisonSolventsItemLinks = {}

    LibRecipe:LoadRecipesFrom(450000)
    LibRecipe:ParseTable()

    --[[
    local LEQ = LibExecutionQueue:new()
    LEQ:addTask(function() LibRecipe.loadRecipesFrom(1, 450000) end, 'Search Items')
    LEQ:addTask(function() LibRecipe.BuildEnchantingRecipes(1, 1, 0) end, 'Enchanting Recipes')
    LEQ:start()
    ]]--
  end
end
